@@ -115,25 +115,15 @@ impl Display for Bson {
             Bson::Document(ref doc) => write!(fmt, "{}", doc),
             Bson::Boolean(b) => write!(fmt, "{}", b),
             Bson::Null => write!(fmt, "null"),
-            Bson::RegularExpression(Regex {
-                ref pattern,
-                ref options,
-            }) => write!(fmt, "/{}/{}", pattern, options),
+            Bson::RegularExpression(ref x) => write!(fmt, "{}", x),
             Bson::JavaScriptCode(ref code)
             | Bson::JavaScriptCodeWithScope(JavaScriptCodeWithScope { ref code, .. }) => {
                 fmt.write_str(code)
             }
             Bson::Int32(i) => write!(fmt, "{}", i),
             Bson::Int64(i) => write!(fmt, "{}", i),
-            Bson::Timestamp(Timestamp { time, increment }) => {
-                write!(fmt, "Timestamp({}, {})", time, increment)
-            }
-            Bson::Binary(Binary { subtype, ref bytes }) => write!(
-                fmt,
-                "Binary({:#x}, {})",
-                u8::from(subtype),
-                base64::encode(bytes)
-            ),
+            Bson::Timestamp(ref x) => write!(fmt, "{}", x),
+            Bson::Binary(ref x) => write!(fmt, "{}", x),
             Bson::ObjectId(ref id) => write!(fmt, "ObjectId(\"{}\")", id),
             Bson::DateTime(date_time) => write!(fmt, "DateTime(\"{}\")", date_time),
             Bson::Symbol(ref sym) => write!(fmt, "Symbol(\"{}\")", sym),
@@ -389,10 +379,12 @@ impl Bson {
             }
             Bson::Double(v) => json!(v),
             Bson::String(v) => json!(v),
-            Bson::Array(v) => json!(v),
-            Bson::Document(v) => {
-                Value::Object(v.into_iter().map(|(k, v)| (k, Value::from(v))).collect())
-            }
+            Bson::Array(v) => Value::Array(v.into_iter().map(Bson::into_relaxed_extjson).collect()),
+            Bson::Document(v) => Value::Object(
+                v.into_iter()
+                    .map(|(k, v)| (k, v.into_relaxed_extjson()))
+                    .collect(),
+            ),
             Bson::Boolean(v) => json!(v),
             Bson::Null => Value::Null,
             Bson::RegularExpression(Regex { pattern, options }) => {
@@ -433,7 +425,7 @@ impl Bson {
             Bson::ObjectId(v) => json!({"$oid": v.to_hex()}),
             Bson::DateTime(v) if v.timestamp_millis() >= 0 && v.to_chrono().year() <= 99999 => {
                 json!({
-                    "$date": v.to_rfc3339(),
+                    "$date": v.to_rfc3339_string(),
                 })
             }
             Bson::DateTime(v) => json!({
@@ -583,7 +575,7 @@ impl Bson {
             }
             Bson::DateTime(v) if v.timestamp_millis() >= 0 && v.to_chrono().year() <= 9999 => {
                 doc! {
-                    "$date": v.to_rfc3339(),
+                    "$date": v.to_rfc3339_string(),
                 }
             }
             Bson::DateTime(v) => doc! {
@@ -740,13 +732,10 @@ impl Bson {
                 if let Ok(regex) = doc.get_document("$regularExpression") {
                     if let Ok(pattern) = regex.get_str("pattern") {
                         if let Ok(options) = regex.get_str("options") {
-                            let mut options: Vec<_> = options.chars().collect();
-                            options.sort_unstable();
-
-                            return Bson::RegularExpression(Regex {
-                                pattern: pattern.into(),
-                                options: options.into_iter().collect(),
-                            });
+                            return Bson::RegularExpression(Regex::new(
+                                pattern.into(),
+                                options.into(),
+                            ));
                         }
                     }
                 }
@@ -984,6 +973,12 @@ pub struct Timestamp {
     pub increment: u32,
 }
 
+impl Display for Timestamp {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        write!(fmt, "Timestamp({}, {})", self.time, self.increment)
+    }
+}
+
 impl Timestamp {
     pub(crate) fn to_le_i64(self) -> i64 {
         let upper = (self.time.to_le() as u64) << 32;
@@ -1018,6 +1013,21 @@ pub struct Regex {
     pub options: String,
 }
 
+impl Regex {
+    pub(crate) fn new(pattern: String, options: String) -> Self {
+        let mut chars: Vec<_> = options.chars().collect();
+        chars.sort_unstable();
+        let options: String = chars.into_iter().collect();
+        Self { pattern, options }
+    }
+}
+
+impl Display for Regex {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        write!(fmt, "/{}/{}", self.pattern, self.options)
+    }
+}
+
 /// Represents a BSON code with scope value.
 #[derive(Debug, Clone, PartialEq)]
 pub struct JavaScriptCodeWithScope {
@@ -1025,6 +1035,12 @@ pub struct JavaScriptCodeWithScope {
     pub scope: Document,
 }
 
+impl Display for JavaScriptCodeWithScope {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        fmt.write_str(&self.code)
+    }
+}
+
 /// Represents a BSON binary value.
 #[derive(Debug, Clone, PartialEq)]
 pub struct Binary {
@@ -1035,32 +1051,43 @@ pub struct Binary {
     pub bytes: Vec<u8>,
 }
 
-impl Binary {
-    fn from_extended_doc(doc: &Document) -> Option<Self> {
-        let binary = doc.get_document("$binary").ok()?;
-        let bytes = binary.get_str("base64").ok()?;
-        let bytes = base64::decode(bytes).ok()?;
-        let subtype = binary.get_str("subType").ok()?;
-        let subtype = hex::decode(subtype).ok()?;
-
-        if subtype.len() == 1 {
-            Some(Self {
-                bytes,
-                subtype: subtype[0].into(),
-            })
-        } else {
-            None
-        }
+impl Display for Binary {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            fmt,
+            "Binary({:#x}, {})",
+            u8::from(self.subtype),
+            base64::encode(&self.bytes)
+        )
     }
 }
 
-#[cfg(feature = "uuid-0_8")]
-#[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
-impl From<uuid::Uuid> for Binary {
-    fn from(uuid: uuid::Uuid) -> Self {
-        Binary {
-            subtype: BinarySubtype::Uuid,
-            bytes: uuid.as_bytes().to_vec(),
+impl Binary {
+    pub(crate) fn from_extended_doc(doc: &Document) -> Option<Self> {
+        let binary_doc = doc.get_document("$binary").ok()?;
+
+        if let Ok(bytes) = binary_doc.get_str("base64") {
+            let bytes = base64::decode(bytes).ok()?;
+            let subtype = binary_doc.get_str("subType").ok()?;
+            let subtype = hex::decode(subtype).ok()?;
+            if subtype.len() == 1 {
+                Some(Self {
+                    bytes,
+                    subtype: subtype[0].into(),
+                })
+            } else {
+                None
+            }
+        } else {
+            // in non-human-readable mode, RawBinary will serialize as
+            // { "$binary": { "bytes": <bytes>, "subType": <i32> } };
+            let binary = binary_doc.get_binary_generic("bytes").ok()?;
+            let subtype = binary_doc.get_i32("subType").ok()?;
+
+            Some(Self {
+                bytes: binary.clone(),
+                subtype: u8::try_from(subtype).ok()?.into(),
+            })
         }
     }
 }
