@@ -1,5 +1,6 @@
 use serde::ser::{
     self,
+    Error as SerdeError,
     Serialize,
     SerializeMap,
     SerializeSeq,
@@ -12,15 +13,18 @@ use serde::ser::{
 use serde_bytes::Bytes;
 
 use crate::{
-    bson::{Array, Binary, Bson, DbPointer, Document, JavaScriptCodeWithScope, Regex, Timestamp},
+    bson::{Array, Bson, DbPointer, Document, JavaScriptCodeWithScope, Regex, Timestamp},
     datetime::DateTime,
     extjson,
     oid::ObjectId,
+    raw::{RawDbPointer, RawRegex, RAW_ARRAY_NEWTYPE, RAW_DOCUMENT_NEWTYPE},
     spec::BinarySubtype,
+    uuid::UUID_NEWTYPE_NAME,
+    Binary,
     Decimal128,
 };
 
-use super::{to_bson, Error};
+use super::{to_bson_with_options, Error};
 
 impl Serialize for ObjectId {
     #[inline]
@@ -106,13 +110,58 @@ impl Serialize for Bson {
 
 /// Serde Serializer
 #[non_exhaustive]
-pub struct Serializer;
+pub struct Serializer {
+    options: SerializerOptions,
+}
+
+/// Options used to configure a [`Serializer`].
+#[derive(Debug, Clone, Default)]
+#[non_exhaustive]
+pub struct SerializerOptions {
+    /// Whether the [`Serializer`] should present itself as human readable or not.
+    /// The default value is true.
+    pub human_readable: Option<bool>,
+}
+
+impl SerializerOptions {
+    /// Create a builder used to construct a new [`SerializerOptions`].
+    pub fn builder() -> SerializerOptionsBuilder {
+        SerializerOptionsBuilder {
+            options: Default::default(),
+        }
+    }
+}
+
+/// A builder used to construct new [`SerializerOptions`] structs.
+pub struct SerializerOptionsBuilder {
+    options: SerializerOptions,
+}
+
+impl SerializerOptionsBuilder {
+    /// Set the value for [`SerializerOptions::is_human_readable`].
+    pub fn human_readable(mut self, value: impl Into<Option<bool>>) -> Self {
+        self.options.human_readable = value.into();
+        self
+    }
+
+    /// Consume this builder and produce a [`SerializerOptions`].
+    pub fn build(self) -> SerializerOptions {
+        self.options
+    }
+}
 
 impl Serializer {
     /// Construct a new `Serializer`.
     #[allow(clippy::new_without_default)]
     pub fn new() -> Serializer {
-        Serializer
+        Serializer {
+            options: Default::default(),
+        }
+    }
+
+    /// Construct a new `Serializer` configured with the provided [`SerializerOptions`].
+    pub fn new_with_options(options: SerializerOptions) -> Self {
+        Serializer { options }
     }
 }
 
@@ -248,13 +297,59 @@ impl ser::Serializer for Serializer {
     #[inline]
     fn serialize_newtype_struct<T: ?Sized>(
         self,
-        _name: &'static str,
+        name: &'static str,
         value: &T,
     ) -> crate::ser::Result<Bson>
     where
         T: Serialize,
     {
-        value.serialize(self)
+        match name {
+            UUID_NEWTYPE_NAME => {
+                let is_human_readable = self.is_human_readable();
+                match value.serialize(self)? {
+                    Bson::String(s) if is_human_readable => {
+                        // the serializer reports itself as human readable, so `Uuid` will
+                        // serialize itself as a string.
+                        let uuid = crate::Uuid::parse_str(s).map_err(Error::custom)?;
+                        Ok(Bson::Binary(uuid.into()))
+                    }
+                    Bson::Binary(b) if !is_human_readable => Ok(Bson::Binary(Binary {
+                        bytes: b.bytes,
+                        subtype: BinarySubtype::Uuid,
+                    })),
+                    b => {
+                        let expectation = if is_human_readable {
+                            "a string"
+                        } else {
+                            "bytes"
+                        };
+                        Err(Error::custom(format!(
+                            "expected UUID to be serialized as {} but got {:?} instead",
+                            expectation, b
+                        )))
+                    }
+                }
+            }
+            // when in non-human-readable mode, raw document / raw array will serialize as bytes.
+            RAW_DOCUMENT_NEWTYPE | RAW_ARRAY_NEWTYPE if !self.is_human_readable() => match value
+                .serialize(self)?
+            {
+                Bson::Binary(b) => {
+                    let doc = Document::from_reader(b.bytes.as_slice()).map_err(Error::custom)?;
+
+                    if name == RAW_DOCUMENT_NEWTYPE {
+                        Ok(Bson::Document(doc))
+                    } else {
+                        Ok(Bson::Array(doc.into_iter().map(|kvp| kvp.1).collect()))
+                    }
+                }
+                b => Err(Error::custom(format!(
+                    "expected raw document or array to be serialized as bytes but got {:?} instead",
+                    b
+                ))),
+            },
+            _ => value.serialize(self),
+        }
     }
 
     #[inline]
@@ -269,7 +364,7 @@ impl ser::Serializer for Serializer {
         T: Serialize,
     {
         let mut newtype_variant = Document::new();
-        newtype_variant.insert(variant, to_bson(value)?);
+        newtype_variant.insert(variant, to_bson_with_options(value, self.options)?);
         Ok(newtype_variant.into())
     }
 
@@ -277,6 +372,7 @@ impl ser::Serializer for Serializer {
     fn serialize_seq(self, len: Option<usize>) -> crate::ser::Result<Self::SerializeSeq> {
         Ok(ArraySerializer {
             inner: Array::with_capacity(len.unwrap_or(0)),
+            options: self.options,
         })
     }
 
@@ -284,6 +380,7 @@ impl ser::Serializer for Serializer {
     fn serialize_tuple(self, len: usize) -> crate::ser::Result<Self::SerializeTuple> {
         Ok(TupleSerializer {
             inner: Array::with_capacity(len),
+            options: self.options,
         })
     }
 
@@ -295,6 +392,7 @@ impl ser::Serializer for Serializer {
     ) -> crate::ser::Result<Self::SerializeTupleStruct> {
         Ok(TupleStructSerializer {
             inner: Array::with_capacity(len),
+            options: self.options,
         })
     }
 
@@ -309,6 +407,7 @@ impl ser::Serializer for Serializer {
         Ok(TupleVariantSerializer {
             inner: Array::with_capacity(len),
             name: variant,
+            options: self.options,
         })
     }
 
@@ -317,6 +416,7 @@ impl ser::Serializer for Serializer {
         Ok(MapSerializer {
             inner: Document::new(),
             next_key: None,
+            options: self.options,
         })
     }
 
@@ -328,6 +428,7 @@ impl ser::Serializer for Serializer {
     ) -> crate::ser::Result<Self::SerializeStruct> {
         Ok(StructSerializer {
             inner: Document::new(),
+            options: self.options,
         })
     }
 
@@ -342,13 +443,19 @@ impl ser::Serializer for Serializer {
         Ok(StructVariantSerializer {
             name: variant,
             inner: Document::new(),
+            options: self.options,
         })
     }
+
+    fn is_human_readable(&self) -> bool {
+        self.options.human_readable.unwrap_or(true)
+    }
 }
 
 #[doc(hidden)]
 pub struct ArraySerializer {
     inner: Array,
+    options: SerializerOptions,
 }
 
 impl SerializeSeq for ArraySerializer {
@@ -356,7 +463,8 @@ impl SerializeSeq for ArraySerializer {
     type Error = Error;
 
     fn serialize_element<T: ?Sized + Serialize>(&mut self, value: &T) -> crate::ser::Result<()> {
-        self.inner.push(to_bson(value)?);
+        self.inner
+            .push(to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -368,6 +476,7 @@ impl SerializeSeq for ArraySerializer {
 #[doc(hidden)]
 pub struct TupleSerializer {
     inner: Array,
+    options: SerializerOptions,
 }
 
 impl SerializeTuple for TupleSerializer {
@@ -375,7 +484,8 @@ impl SerializeTuple for TupleSerializer {
     type Error = Error;
 
     fn serialize_element<T: ?Sized + Serialize>(&mut self, value: &T) -> crate::ser::Result<()> {
-        self.inner.push(to_bson(value)?);
+        self.inner
+            .push(to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -387,6 +497,7 @@ impl SerializeTuple for TupleSerializer {
 #[doc(hidden)]
 pub struct TupleStructSerializer {
     inner: Array,
+    options: SerializerOptions,
 }
 
 impl SerializeTupleStruct for TupleStructSerializer {
@@ -394,7 +505,8 @@ impl SerializeTupleStruct for TupleStructSerializer {
     type Error = Error;
 
     fn serialize_field<T: ?Sized + Serialize>(&mut self, value: &T) -> crate::ser::Result<()> {
-        self.inner.push(to_bson(value)?);
+        self.inner
+            .push(to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -407,6 +519,7 @@ impl SerializeTupleStruct for TupleStructSerializer {
 pub struct TupleVariantSerializer {
     inner: Array,
     name: &'static str,
+    options: SerializerOptions,
 }
 
 impl SerializeTupleVariant for TupleVariantSerializer {
@@ -414,7 +527,8 @@ impl SerializeTupleVariant for TupleVariantSerializer {
     type Error = Error;
 
     fn serialize_field<T: ?Sized + Serialize>(&mut self, value: &T) -> crate::ser::Result<()> {
-        self.inner.push(to_bson(value)?);
+        self.inner
+            .push(to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -429,6 +543,7 @@ impl SerializeTupleVariant for TupleVariantSerializer {
 pub struct MapSerializer {
     inner: Document,
     next_key: Option<String>,
+    options: SerializerOptions,
 }
 
 impl SerializeMap for MapSerializer {
@@ -436,7 +551,7 @@ impl SerializeMap for MapSerializer {
     type Error = Error;
 
     fn serialize_key<T: ?Sized + Serialize>(&mut self, key: &T) -> crate::ser::Result<()> {
-        self.next_key = match to_bson(&key)? {
+        self.next_key = match to_bson_with_options(&key, self.options.clone())? {
             Bson::String(s) => Some(s),
             other => return Err(Error::InvalidDocumentKey(other)),
         };
@@ -445,7 +560,8 @@ impl SerializeMap for MapSerializer {
 
     fn serialize_value<T: ?Sized + Serialize>(&mut self, value: &T) -> crate::ser::Result<()> {
         let key = self.next_key.take().unwrap_or_default();
-        self.inner.insert(key, to_bson(&value)?);
+        self.inner
+            .insert(key, to_bson_with_options(&value, self.options.clone())?);
         Ok(())
     }
 
@@ -457,6 +573,7 @@ impl SerializeMap for MapSerializer {
 #[doc(hidden)]
 pub struct StructSerializer {
     inner: Document,
+    options: SerializerOptions,
 }
 
 impl SerializeStruct for StructSerializer {
@@ -468,7 +585,8 @@ impl SerializeStruct for StructSerializer {
         key: &'static str,
         value: &T,
     ) -> crate::ser::Result<()> {
-        self.inner.insert(key, to_bson(value)?);
+        self.inner
+            .insert(key, to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -481,6 +599,7 @@ impl SerializeStruct for StructSerializer {
 pub struct StructVariantSerializer {
     inner: Document,
     name: &'static str,
+    options: SerializerOptions,
 }
 
 impl SerializeStructVariant for StructVariantSerializer {
@@ -492,7 +611,8 @@ impl SerializeStructVariant for StructVariantSerializer {
         key: &'static str,
         value: &T,
     ) -> crate::ser::Result<()> {
-        self.inner.insert(key, to_bson(value)?);
+        self.inner
+            .insert(key, to_bson_with_options(value, self.options.clone())?);
         Ok(())
     }
 
@@ -528,13 +648,11 @@ impl Serialize for Regex {
     where
         S: ser::Serializer,
     {
-        let mut state = serializer.serialize_struct("$regularExpression", 1)?;
-        let body = extjson::models::RegexBody {
-            pattern: self.pattern.clone(),
-            options: self.options.clone(),
+        let raw = RawRegex {
+            pattern: self.pattern.as_str(),
+            options: self.options.as_str(),
         };
-        state.serialize_field("$regularExpression", &body)?;
-        state.end()
+        raw.serialize(serializer)
     }
 }
 
@@ -602,12 +720,10 @@ impl Serialize for DbPointer {
     where
         S: ser::Serializer,
     {
-        let mut state = serializer.serialize_struct("$dbPointer", 1)?;
-        let body = extjson::models::DbPointerBody {
-            ref_ns: self.namespace.clone(),
-            id: self.id.into(),
+        let raw = RawDbPointer {
+            namespace: self.namespace.as_str(),
+            id: self.id,
         };
-        state.serialize_field("$dbPointer", &body)?;
-        state.end()
+        raw.serialize(serializer)
     }
 }
